@@ -8,6 +8,7 @@ pub use deno_ast::swc::dep_graph::DependencyKind;
 use anyhow::Result;
 use deno_ast::parse_module;
 use deno_ast::swc::common::comments::Comment;
+use deno_ast::swc::common::comments::CommentKind;
 use deno_ast::swc::common::BytePos;
 use deno_ast::swc::common::Span;
 use deno_ast::Diagnostic;
@@ -27,6 +28,8 @@ lazy_static! {
   static ref DENO_TYPES_RE: Regex =
     Regex::new(r#"(?i)^\s*@deno-types\s*=\s*(?:["']([^"']+)["']|(\S+))"#)
       .unwrap();
+  /// Matches a JSDoc import type reference (`{import("./example.js")}`).
+  static ref JSDOC_IMPORT_RE: Regex = Regex::new(r#"\{\s*import\(['"]([^'"]+)['"]\)[^}]*}"#).unwrap();
   /// Matches the `@jsxImportSource` pragma.
   static ref JSX_IMPORT_SOURCE_RE: Regex = Regex::new(r#"(?i)^[\s*]*@jsxImportSource\s+(\S+)"#).unwrap();
   /// Matches a `/// <reference ... />` comment reference.
@@ -81,6 +84,29 @@ pub fn analyze_deno_types(
   }
 }
 
+/// Searches JSDoc comment blocks for type imports
+/// (e.g. `{import("./types.d.ts").Type}`) and returns a vector of tuples of
+/// the specifier and the span of the import.
+pub fn analyze_jsdoc_imports(
+  parsed_source: &ParsedSource,
+) -> Vec<(String, Span)> {
+  let mut deps = Vec::new();
+  for comment in parsed_source.comments().get_vec().iter() {
+    if comment.kind != CommentKind::Block || !comment.text.starts_with('*') {
+      continue;
+    }
+    for captures in JSDOC_IMPORT_RE.captures_iter(&comment.text) {
+      if let Some(m) = captures.get(1) {
+        deps.push((
+          m.as_str().to_string(),
+          comment_match_to_swc_span(comment, &m),
+        ));
+      }
+    }
+  }
+  deps
+}
+
 /// Searches comments for a `@jsxImportSource` pragma on JSX/TSX media types
 pub fn analyze_jsx_import_sources(
   parsed_source: &ParsedSource,
@@ -225,6 +251,7 @@ impl SourceParser for DefaultSourceParser {
 #[cfg(test)]
 mod tests {
   use super::*;
+  use serde_json::json;
 
   #[test]
   fn test_parse() {
@@ -370,4 +397,66 @@ mod tests {
       Some(&"json".to_string())
     );
   }
+
+  #[test]
+  fn test_analyze_jsdoc_imports() {
+    let specifier = ModuleSpecifier::parse("file:///a/test.js").unwrap();
+    let source = Arc::new(
+      r#"
+/** @module */
+
+/**
+ * Some stuff here
+ * 
+ * @type {import("./a.js").A}
+ */
+const a = "a";
+
+/**
+ * Some other stuff here
+ * 
+ * @param {import('./b.js').C}
+ * @returns {import("./d.js")}
+ */
+function b(c) {
+  return;
+}
+"#
+      .to_string(),
+    );
+    let parser = DefaultSourceParser::new();
+    let result = parser.parse_module(&specifier, source, MediaType::TypeScript);
+    assert!(result.is_ok());
+    let parsed_source = result.unwrap();
+    let dependencies = analyze_jsdoc_imports(&parsed_source);
+    assert_eq!(
+      json!(dependencies),
+      json!([
+        [
+          "./a.js",
+          {
+            "start": 62,
+            "end": 68,
+            "ctxt": 0
+          }
+        ],
+        [
+          "./b.js",
+          {
+            "start": 146,
+            "end": 152,
+            "ctxt": 0
+          }
+        ],
+        [
+          "./d.js",
+          {
+            "start": 179,
+            "end": 185,
+            "ctxt": 0
+          }
+        ]
+      ])
+    );
+  }
 }
