@@ -3,6 +3,7 @@
 use crate::ast;
 use crate::ast::analyze_deno_types;
 use crate::ast::analyze_dependencies;
+use crate::ast::analyze_jsdoc_imports;
 use crate::ast::analyze_jsx_import_sources;
 use crate::ast::analyze_ts_references;
 use crate::ast::SourceParser;
@@ -124,10 +125,16 @@ impl Range {
 #[derive(Debug)]
 pub enum ModuleGraphError {
   InvalidSource(ModuleSpecifier, Option<String>),
+  InvalidTypeAssertion {
+    specifier: ModuleSpecifier,
+    actual_media_type: MediaType,
+    expected_media_type: MediaType,
+  },
   LoadingErr(ModuleSpecifier, Arc<anyhow::Error>),
   Missing(ModuleSpecifier),
   ParseErr(ModuleSpecifier, deno_ast::Diagnostic),
   ResolutionError(ResolutionError),
+  UnsupportedImportAssertionType(ModuleSpecifier, String),
   UnsupportedMediaType(ModuleSpecifier, MediaType),
 }
 
@@ -144,6 +151,18 @@ impl Clone for ModuleGraphError {
       Self::InvalidSource(specifier, maybe_filename) => {
         Self::InvalidSource(specifier.clone(), maybe_filename.clone())
       }
+      Self::InvalidTypeAssertion {
+        specifier,
+        actual_media_type,
+        expected_media_type,
+      } => Self::InvalidTypeAssertion {
+        specifier: specifier.clone(),
+        actual_media_type: *actual_media_type,
+        expected_media_type: *expected_media_type,
+      },
+      Self::UnsupportedImportAssertionType(specifier, kind) => {
+        Self::UnsupportedImportAssertionType(specifier.clone(), kind.clone())
+      }
       Self::UnsupportedMediaType(specifier, media_type) => {
         Self::UnsupportedMediaType(specifier.clone(), *media_type)
       }
@@ -161,7 +180,9 @@ impl ModuleGraphError {
       | Self::ParseErr(s, _)
       | Self::InvalidSource(s, _)
       | Self::UnsupportedMediaType(s, _)
+      | Self::UnsupportedImportAssertionType(s, _)
       | Self::Missing(s) => s,
+      Self::InvalidTypeAssertion { specifier, .. } => specifier,
     }
   }
 }
@@ -174,13 +195,13 @@ impl fmt::Display for ModuleGraphError {
       Self::LoadingErr(_, err) => err.fmt(f),
       Self::ParseErr(_, diagnostic) => write!(f, "The module's source code could not be parsed: {}", diagnostic),
       Self::ResolutionError(err) => err.fmt(f),
-      Self::InvalidSource(specifier, maybe_filename) => if let Some(filename) = maybe_filename {
-        write!(f, "The source code is invalid, as it does not match the expected hash in the lock file.\n  Specifier: {}\n  Lock file: {}", specifier, filename)
-      } else {
-        write!(f, "The source code is invalid, as it does not match the expected hash in the lock file.\n  Specifier: {}", specifier)
-      },
-      Self::UnsupportedMediaType(specifier, media_type) => write!(f, "An unsupported media type was attempted to be imported as a module.\n  Specifier: {}\n  MediaType: {}", specifier, media_type),
-      Self::Missing(specifier) => write!(f, "Cannot load module \"{}\".", specifier),
+      Self::InvalidSource(specifier, Some(filename)) => write!(f, "The source code is invalid, as it does not match the expected hash in the lock file.\n  Specifier: {}\n  Lock file: {}", specifier, filename),
+      Self::InvalidSource(specifier, None) => write!(f, "The source code is invalid, as it does not match the expected hash in the lock file.\n  Specifier: {}", specifier),
+      Self::InvalidTypeAssertion { specifier, actual_media_type, expected_media_type } => write!(f, "Expected a {} module, but identified a {} module.\n  Specifier: {}", expected_media_type, actual_media_type, specifier),
+      Self::UnsupportedMediaType(specifier, MediaType::Json) => write!(f, "Expected a JavaScript or TypeScript module, but identified a Json module. Consider importing Json modules with an import assertion with the type of \"json\".\n  Specifier: {}", specifier),
+      Self::UnsupportedMediaType(specifier, media_type) => write!(f, "Expected a JavaScript or TypeScript module, but identified a {} module. Importing these types of modules is currently not supported.\n  Specifier: {}", media_type, specifier),
+      Self::UnsupportedImportAssertionType(_, kind) => write!(f, "The import assertion type of \"{}\" is unsupported.", kind),
+      Self::Missing(specifier) => write!(f, "Module not found \"{}\".", specifier),
     }
   }
 }
@@ -1084,6 +1105,23 @@ pub(crate) fn parse_module(
     )));
   }
 
+  match maybe_assert_type {
+    Some("json") => {
+      return ModuleSlot::Err(ModuleGraphError::InvalidTypeAssertion {
+        specifier: specifier.clone(),
+        actual_media_type: media_type,
+        expected_media_type: MediaType::Json,
+      })
+    }
+    Some(assert_type) => {
+      return ModuleSlot::Err(ModuleGraphError::UnsupportedImportAssertionType(
+        specifier.clone(),
+        assert_type.to_string(),
+      ))
+    }
+    _ => (),
+  }
+
   // Here we check for known ES Modules that we will analyze the dependencies of
   match &media_type {
     MediaType::JavaScript
@@ -1190,6 +1228,14 @@ pub(crate) fn parse_module_from_ast(
     dep.maybe_code = resolved_dependency;
   }
 
+  // Analyze any JSDoc type imports
+  for (import_source, span) in analyze_jsdoc_imports(parsed_source) {
+    let range = Range::from_swc_span(&module.specifier, parsed_source, &span);
+    let resolved_dependency = resolve(&import_source, &range, maybe_resolver);
+    let dep = module.dependencies.entry(import_source).or_default();
+    dep.maybe_type = resolved_dependency;
+  }
+
   // Analyze the X-TypeScript-Types header
   if module.maybe_types_dependency.is_none() {
     if let Some(headers) = maybe_headers {
@@ -1310,7 +1356,7 @@ pub(crate) struct Builder<'a> {
   loader: &'a mut dyn Loader,
   maybe_resolver: Option<&'a dyn Resolver>,
   pending: FuturesUnordered<LoadFuture>,
-  pending_assert_types: HashMap<ModuleSpecifier, Vec<Option<String>>>,
+  pending_assert_types: HashMap<ModuleSpecifier, HashSet<Option<String>>>,
   dynamic_branches: HashMap<ModuleSpecifier, Option<String>>,
   source_parser: &'a dyn SourceParser,
 }
@@ -1440,7 +1486,7 @@ impl<'a> Builder<'a> {
       .pending_assert_types
       .entry(specifier.clone())
       .or_default();
-    assert_types.push(maybe_assert_type.map(String::from));
+    assert_types.insert(maybe_assert_type.map(String::from));
     if !self.graph.module_slots.contains_key(specifier) {
       self
         .graph
