@@ -525,6 +525,10 @@ console.log(a);
     )
     .await;
     assert!(graph.valid().is_err());
+    assert_eq!(
+      graph.valid().err().unwrap().to_string(),
+      r#"Module not found "file:///a/test02.js"."#
+    );
     assert!(graph.valid_types_only().is_ok());
   }
 
@@ -751,7 +755,7 @@ console.log(a);
             "file:///a/test01.tsx",
             None,
             r#"/* @jsxImportSource https://example.com/preact */
-            
+
             export function A() {
               <div>Hello Deno</div>
             }
@@ -808,7 +812,7 @@ console.log(a);
               }
             ],
             "mediaType": "TSX",
-            "size": 159,
+            "size": 147,
             "specifier": "file:///a/test01.tsx"
           },
           {
@@ -993,6 +997,118 @@ console.log(a);
     );
   }
 
+  #[tokio::test]
+  async fn test_create_graph_with_jsdoc_imports() {
+    let mut loader = setup(
+      vec![
+        (
+          "file:///a/test.js",
+          Ok((
+            "file:///a/test.js",
+            None,
+            r#"
+/**
+ * Some js doc
+ * 
+ * @param {import("./types.d.ts").A} a
+ * @return {import("./other.ts").B}
+ */
+export function a(a) {
+  return;
+}
+"#,
+          )),
+        ),
+        (
+          "file:///a/types.d.ts",
+          Ok(("file:///a/types.d.ts", None, r#"export type A = string;"#)),
+        ),
+        (
+          "file:///a/other.ts",
+          Ok((
+            "file:///a/other.ts",
+            None,
+            r#"export type B = string | undefined;"#,
+          )),
+        ),
+      ],
+      vec![],
+    );
+    let root = ModuleSpecifier::parse("file:///a/test.js").unwrap();
+    let graph = create_graph(
+      vec![root.clone()],
+      false,
+      None,
+      &mut loader,
+      None,
+      None,
+      None,
+    )
+    .await;
+    assert_eq!(
+      json!(graph),
+      json!({
+        "roots": [
+          "file:///a/test.js"
+        ],
+        "modules": [
+          {
+            "dependencies": [],
+            "mediaType": "TypeScript",
+            "size": 35,
+            "specifier": "file:///a/other.ts"
+          },
+          {
+            "dependencies": [
+              {
+                "specifier": "./other.ts",
+                "type": {
+                  "specifier": "file:///a/other.ts",
+                  "span": {
+                    "start": {
+                      "line": 5,
+                      "character": 20
+                    },
+                    "end": {
+                      "line": 5,
+                      "character": 30
+                    }
+                  }
+                }
+              },
+              {
+                "specifier": "./types.d.ts",
+                "type": {
+                  "specifier": "file:///a/types.d.ts",
+                  "span": {
+                    "start": {
+                      "line": 4,
+                      "character": 19
+                    },
+                    "end": {
+                      "line": 4,
+                      "character": 31
+                    }
+                  }
+                }
+              }
+            ],
+            "mediaType": "JavaScript",
+            "size": 138,
+            "specifier": "file:///a/test.js"
+          },
+          {
+            "dependencies": [],
+            "mediaType": "Dts",
+            "size": 23,
+            "specifier": "file:///a/types.d.ts"
+          }
+        ],
+        "redirects": {}
+      })
+    );
+  }
+
   #[tokio::test]
   async fn test_create_graph_with_redirects() {
     let mut loader = setup(
@@ -1557,6 +1673,180 @@ console.log(a);
     );
   }
 
+  #[tokio::test]
+  async fn test_create_graph_import_assertion_errors() {
+    let mut loader = setup(
+      vec![
+        (
+          "file:///a/test01.ts",
+          Ok((
+            "file:///a/test01.ts",
+            None,
+            r#"
+            import a from "./a.json";
+            import b from "./b.json" assert { type: "json" };
+            import c from "./c.js" assert { type: "json" };
+            import d from "./d.json" assert { type: "css" };
+            import e from "./e.wasm";
+            "#,
+          )),
+        ),
+        (
+          "file:///a/a.json",
+          Ok(("file:///a/a.json", None, r#"{"a":"b"}"#)),
+        ),
+        (
+          "file:///a/b.json",
+          Ok(("file:///a/b.json", None, r#"{"a":"b"}"#)),
+        ),
+        (
+          "file:///a/c.js",
+          Ok(("file:///a/c.js", None, r#"export const c = "c";"#)),
+        ),
+        (
+          "file:///a/d.json",
+          Ok(("file:///a/d.json", None, r#"{"a":"b"}"#)),
+        ),
+        ("file:///a/e.wasm", Ok(("file:///a/e.wasm", None, ""))),
+      ],
+      vec![],
+    );
+    let root_specifier =
+      ModuleSpecifier::parse("file:///a/test01.ts").expect("bad url");
+    let graph = create_graph(
+      vec![root_specifier.clone()],
+      false,
+      None,
+      &mut loader,
+      None,
+      None,
+      None,
+    )
+    .await;
+    assert_eq!(
+      json!(graph),
+      json!({
+        "roots": [
+          "file:///a/test01.ts"
+        ],
+        "modules": [
+          {
+            "specifier": "file:///a/a.json",
+            "error": "Expected a JavaScript or TypeScript module, but identified a Json module. Consider importing Json modules with an import assertion with the type of \"json\".\n  Specifier: file:///a/a.json"
+          },
+          {
+            "size": 9,
+            "mediaType": "Json",
+            "specifier": "file:///a/b.json"
+          },
+          {
+            "specifier": "file:///a/c.js",
+            "error": "Expected a Json module, but identified a JavaScript module.\n  Specifier: file:///a/c.js"
+          },
+          {
+            "specifier": "file:///a/d.json",
+            "error": "The import assertion type of \"css\" is unsupported."
+          },
+          {
+            "specifier": "file:///a/e.wasm",
+            "error": "Expected a JavaScript or TypeScript module, but identified a Wasm module. Importing these types of modules is currently not supported.\n  Specifier: file:///a/e.wasm"
+          },
+          {
+            "dependencies": [
+              {
+                "specifier": "./a.json",
+                "code": {
+                  "specifier": "file:///a/a.json",
+                  "span": {
+                    "start": {
+                      "line": 1,
+                      "character": 26
+                    },
+                    "end": {
+                      "line": 1,
+                      "character": 36
+                    }
+                  }
+                }
+              },
+              {
+                "specifier": "./b.json",
+                "code": {
+                  "specifier": "file:///a/b.json",
+                  "span": {
+                    "start": {
+                      "line": 2,
+                      "character": 26
+                    },
+                    "end": {
+                      "line": 2,
+                      "character": 36
+                    }
+                  }
+                },
+                "assertionType": "json"
+              },
+              {
+                "specifier": "./c.js",
+                "code": {
+                  "specifier": "file:///a/c.js",
+                  "span": {
+                    "start": {
+                      "line": 3,
+                      "character": 26
+                    },
+                    "end": {
+                      "line": 3,
+                      "character": 34
+                    }
+                  }
+                },
+                "assertionType": "json"
+              },
+              {
+                "specifier": "./d.json",
+                "code": {
+                  "specifier": "file:///a/d.json",
+                  "span": {
+                    "start": {
+                      "line": 4,
+                      "character": 26
+                    },
+                    "end": {
+                      "line": 4,
+                      "character": 36
+                    }
+                  }
+                },
+                "assertionType": "css"
+              },
+              {
+                "specifier": "./e.wasm",
+                "code": {
+                  "specifier": "file:///a/e.wasm",
+                  "span": {
+                    "start": {
+                      "line": 5,
+                      "character": 26
+                    },
+                    "end": {
+                      "line": 5,
+                      "character": 36
+                    }
+                  }
+                }
+              }
+            ],
+            "mediaType": "TypeScript",
+            "size": 272,
+            "specifier": "file:///a/test01.ts"
+          }
+        ],
+        "redirects": {}
+      })
+    );
+  }
+
   #[test]
   fn test_parse_module() {
     let specifier = ModuleSpecifier::parse("file:///a/test01.ts").unwrap();
@@ -1716,4 +2006,73 @@ console.log(a);
     );
     assert!(result.is_ok());
   }
+
+  #[test]
+  fn test_parse_module_with_jsdoc_imports() {
+    let specifier = ModuleSpecifier::parse("file:///a/test.js").unwrap();
+    let result = parse_module(
+      &specifier,
+      None,
+      Arc::new(
+        r#"
+/**
+ * Some js doc
+ * 
+ * @param {import("./types.d.ts").A} a
+ * @return {import("./other.ts").B}
+ */
+export function a(a) {
+  return;
+}
+"#
+        .to_string(),
+      ),
+      None,
+      None,
+    );
+    assert!(result.is_ok());
+    let actual = result.unwrap();
+    assert_eq!(
+      json!(actual),
+      json!({
+        "dependencies": [
+          {
+            "specifier": "./other.ts",
+            "type": {
+              "specifier": "file:///a/other.ts",
+              "span": {
+                "start": {
+                  "line": 5,
+                  "character": 20,
+                },
+                "end": {
+                  "line": 5,
+                  "character": 30
+                }
+              }
+            }
+          },
+          {
+            "specifier": "./types.d.ts",
+            "type": {
+              "specifier": "file:///a/types.d.ts",
+              "span": {
+                "start": {
+                  "line": 4,
+                  "character": 19,
+                },
+                "end": {
+                  "line": 4,
+                  "character": 31,
+                }
+              }
+            }
+          }
+        ],
+        "mediaType": "JavaScript",
+        "size": 138,
+        "specifier": "file:///a/test.js"
+      })
+    );
+  }
 }
